import unittest

from django.conf import settings
from httplib2 import ServerNotFoundError
from mock import patch
from clickreviews.utilities import verify_click_package_published


class VerifyPackageTestCase(unittest.TestCase):

    def setUp(self):
        super(VerifyPackageTestCase, self).setUp()
        self.mock_request = self.patch_httplib2_request()
        self.patch_click_api_uri()

    def patch_httplib2_request(self):
        patcher = patch('clickreviews.utilities.Http.request')
        self.addCleanup(patcher.stop)
        return patcher.start()

    def patch_click_api_uri(self, uri="http://example.com/api/"):
        patcher = patch.multiple(settings, CLICK_INDEX_API_URI=uri)
        self.addCleanup(patcher.stop)
        return patcher.start()

    def test_click_verify(self):
        self.mock_request.return_value = ({
            'content-length': '1152',
            'status': '200',
        }, '')

        result = verify_click_package_published(
            "com.ubuntu.developer.dholbach.webapp-bvg")

        self.assertTrue(result)
        self.mock_request.assert_called_once_with(
            "http://example.com/api/package/"
            "com.ubuntu.developer.dholbach.webapp-bvg", "HEAD"
        )

    def test_click_verify_false(self):
        self.mock_request.return_value = ({
            'content-length': '1152',
            'status': '404',
        }, '')

        result = verify_click_package_published(
            "com.ubuntu.developer.dholbach.webapp-bvg")

        self.assertFalse(result)
        self.mock_request.assert_called_once_with(
            "http://example.com/api/package/"
            "com.ubuntu.developer.dholbach.webapp-bvg", "HEAD"
        )

    def test_click_verify_error(self):
        self.mock_request.side_effect = ServerNotFoundError

        result = verify_click_package_published(
            "com.ubuntu.developer.dholbach.webapp-bvg")

        self.assertFalse(result)
        self.mock_request.assert_called_once_with(
            "http://example.com/api/package/"
            "com.ubuntu.developer.dholbach.webapp-bvg", "HEAD"
        )

    def test_timeout_is_set_from_settings(self):
        expected_timeout = 1
        patcher = patch.multiple(
            settings, CLICK_INDEX_TIMEOUT_SECONDS=expected_timeout)
        self.addCleanup(patcher.stop)
        patcher.start()

        patcher = patch('clickreviews.utilities.Http')
        self.addCleanup(patcher.stop)
        mock_http = patcher.start()

        verify_click_package_published(
            "com.ubuntu.developer.dholbach.webapp-bvg")

        mock_http.assert_called_once_with(timeout=expected_timeout)
